@@ -2,23 +2,21 @@
 # this repository contains the full copyright notices and license terms.
 "Sales extension for managing leads and opportunities"
 import datetime
+from itertools import groupby
 
-from sql import Literal, Null
-from sql.aggregate import Max, Count, Sum
-from sql.conditionals import Case, Coalesce
-from sql.functions import Extract
+from sql.functions import CharLength
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, Workflow, fields, \
-    sequence_ordered
-from trytond.model.exceptions import AccessError
-from trytond.pyson import Eval, In, If, Get, Bool
-from trytond.transaction import Transaction
-from trytond.pool import Pool
-
 from trytond.ir.attachment import AttachmentCopyMixin
 from trytond.ir.note import NoteCopyMixin
+from trytond.model import (
+    Index, ModelSQL, ModelView, Workflow, fields, sequence_ordered)
+from trytond.model.exceptions import AccessError
 from trytond.modules.company.model import employee_field, set_employee
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool
+from trytond.pyson import Bool, Eval, Get, If, In
+from trytond.transaction import Transaction
 
 
 class SaleOpportunity(
@@ -32,17 +30,15 @@ class SaleOpportunity(
     _states_start = {
         'readonly': Eval('state') != 'lead',
         }
-    _depends_start = ['state']
     _states_stop = {
         'readonly': Eval('state').in_(
             ['converted', 'won', 'lost', 'cancelled']),
     }
-    _depends_stop = ['state']
 
-    number = fields.Char('Number', readonly=True, required=True, select=True)
-    reference = fields.Char('Reference', select=True)
+    number = fields.Char("Number", readonly=True, required=True)
+    reference = fields.Char("Reference")
     party = fields.Many2One(
-        'party.party', "Party", select=True,
+        'party.party', "Party",
         states={
             'readonly': Eval('state').in_(['converted', 'lost', 'cancelled']),
             'required': ~Eval('state').in_(['lead', 'lost', 'cancelled']),
@@ -50,7 +46,7 @@ class SaleOpportunity(
         context={
             'company': Eval('company', -1),
             },
-        depends=['state', 'company'])
+        depends={'company'})
     contact = fields.Many2One(
         'party.contact_mechanism', "Contact",
         context={
@@ -60,50 +56,42 @@ class SaleOpportunity(
             'related_party': Eval('party'),
             },
         depends=['party', 'company'])
-    address = fields.Many2One('party.address', 'Address',
-        domain=[('party', '=', Eval('party'))],
-        select=True, depends=['party', 'state'],
-        states=_states_stop)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True,
+    address = fields.Many2One(
+        'party.address', "Address", states=_states_stop,
+        domain=[('party', '=', Eval('party'))])
+    company = fields.Many2One(
+        'company.company', "Company", required=True,
         states={
             'readonly': _states_stop['readonly'] | Eval('party', True),
             },
         domain=[
             ('id', If(In('company', Eval('context', {})), '=', '!='),
                 Get(Eval('context', {}), 'company', 0)),
-            ],
-        depends=_depends_stop)
-    currency = fields.Function(fields.Many2One('currency.currency',
-        'Currency'), 'get_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-            'get_currency_digits')
-    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
-        states=_states_stop, depends=_depends_stop + ['currency_digits'],
+            ])
+    currency = fields.Many2One(
+        'currency.currency', "Currency", required=True, states=_states_start)
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency',
+        states=_states_stop,
         help='Estimated revenue amount.')
-    payment_term = fields.Many2One('account.invoice.payment_term',
-        'Payment Term', states={
+    payment_term = fields.Many2One(
+        'account.invoice.payment_term', "Payment Term", ondelete='RESTRICT',
+        states={
             'readonly': In(Eval('state'),
                 ['converted', 'lost', 'cancelled']),
-            },
-        depends=['state'])
+            })
     employee = fields.Many2One('company.employee', 'Employee',
         states={
             'readonly': _states_stop['readonly'],
             'required': ~Eval('state').in_(['lead', 'lost', 'cancelled']),
         },
-        depends=['state', 'company'],
         domain=[('company', '=', Eval('company'))])
-    start_date = fields.Date('Start Date', required=True, select=True,
-        states=_states_start, depends=_depends_start)
-    end_date = fields.Date('End Date', select=True,
-        states=_states_stop, depends=_depends_stop)
-    description = fields.Char('Description',
-        states=_states_stop, depends=_depends_stop)
-    comment = fields.Text('Comment', states=_states_stop,
-        depends=_depends_stop)
+    start_date = fields.Date("Start Date", required=True, states=_states_start)
+    end_date = fields.Date("End Date", states=_states_stop)
+    description = fields.Char('Description', states=_states_stop)
+    comment = fields.Text('Comment', states=_states_stop)
     lines = fields.One2Many('sale.opportunity.line', 'opportunity', 'Lines',
-        states=_states_stop, depends=_depends_stop)
+        states=_states_stop)
     conversion_probability = fields.Float('Conversion Probability',
         digits=(1, 4), required=True,
         domain=[
@@ -114,10 +102,10 @@ class SaleOpportunity(
             'readonly': ~Eval('state').in_(
                 ['opportunity', 'lead', 'converted']),
             },
-        depends=['state'], help="Percentage between 0 and 100.")
+        help="Percentage between 0 and 100.")
     lost_reason = fields.Text('Reason for loss', states={
             'invisible': Eval('state') != 'lost',
-            }, depends=['state'])
+            })
     sales = fields.One2Many('sale.sale', 'origin', 'Sales')
 
     converted_by = employee_field(
@@ -127,64 +115,56 @@ class SaleOpportunity(
             ('opportunity', "Opportunity"),
             ('converted', "Converted"),
             ('won', "Won"),
-            ('cancelled', "Cancelled"),
             ('lost', "Lost"),
-            ], "State", required=True, select=True,
-        sort=False, readonly=True)
+            ('cancelled', "Cancelled"),
+            ], "State", required=True, sort=False, readonly=True)
 
-    del _states_start, _depends_start
-    del _states_stop, _depends_stop
+    del _states_start
+    del _states_stop
 
     @classmethod
     def __register__(cls, module_name):
         pool = Pool()
-        Sale = pool.get('sale.sale')
+        Company = pool.get('company.company')
         transaction = Transaction()
         cursor = transaction.connection.cursor()
-        update = transaction.connection.cursor()
         sql_table = cls.__table__()
-        sale = Sale.__table__()
+        company = Company.__table__()
 
         table = cls.__table_handler__(module_name)
-        number_exists = table.column_exist('number')
-
-        # Migration from 3.8: rename reference into number
-        if table.column_exist('reference') and not number_exists:
-            table.column_rename('reference', 'number')
-            number_exists = True
+        currency_exists = table.column_exist('currency')
 
         super(SaleOpportunity, cls).__register__(module_name)
-        table = cls.__table_handler__(module_name)
-
-        # Migration from 3.4: replace sale by origin
-        if table.column_exist('sale'):
-            cursor.execute(*sql_table.select(
-                    sql_table.id, sql_table.sale,
-                    where=sql_table.sale != Null))
-            for id_, sale_id in cursor:
-                update.execute(*sale.update(
-                        columns=[sale.origin],
-                        values=['%s,%s' % (cls.__name__, id_)],
-                        where=sale.id == sale_id))
-            table.drop_column('sale')
-
-        # Migration from 4.0: change probability into conversion probability
-        if table.column_exist('probability'):
-            cursor.execute(*sql_table.update(
-                    [sql_table.conversion_probability],
-                    [sql_table.probability / 100.0]))
-            table.drop_constraint('check_percentage')
-            table.drop_column('probability')
-
-        # Migration from 4.2: make employee not required
-        table.not_null_action('employee', action='remove')
 
         # Migration from 5.0: drop required on description
         table.not_null_action('description', action='remove')
 
+        # Migration from 6.4: store currency
+        if not currency_exists:
+            value = company.select(
+                company.currency,
+                where=(sql_table.company == company.id))
+            cursor.execute(*sql_table.update(
+                    [sql_table.currency],
+                    [value]))
+
     @classmethod
     def __setup__(cls):
+        cls.number.search_unaccented = False
+        cls.reference.search_unaccented = False
         super(SaleOpportunity, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.update({
+                Index(t, (t.reference, Index.Similarity())),
+                Index(t, (t.party, Index.Equality())),
+                Index(
+                    t,
+                    (t.start_date, Index.Range(order='DESC')),
+                    (t.end_date, Index.Range(order='DESC'))),
+                Index(
+                    t, (t.state, Index.Equality()),
+                    where=t.state.in_(['lead', 'opportunity'])),
+                })
         cls._order.insert(0, ('start_date', 'DESC'))
         cls._transitions |= set((
                 ('lead', 'opportunity'),
@@ -211,6 +191,14 @@ class SaleOpportunity(
                     'depends': ['state'],
                     },
                 'opportunity': {
+                    'pre_validate': [
+                        If(~Eval('party'),
+                            ('party', '!=', None),
+                            ()),
+                        If(~Eval('employee'),
+                            ('employee', '!=', None),
+                            ()),
+                        ],
                     'invisible': ~Eval('state').in_(['lead']),
                     'depends': ['state'],
                     },
@@ -228,6 +216,11 @@ class SaleOpportunity(
                     },
                 })
 
+    @classmethod
+    def order_number(cls, tables):
+        table, _ = tables[None]
+        return [CharLength(table.number), table.number]
+
     @staticmethod
     def default_state():
         return 'lead'
@@ -245,16 +238,55 @@ class SaleOpportunity(
     def default_company():
         return Transaction().context.get('company')
 
+    @fields.depends('amount', 'company')
+    def on_change_company(self):
+        self.payment_term = self.default_payment_term(
+            company=self.company.id if self.company else None)
+        if not self.amount:
+            self.currency = self.default_currency(
+                company=self.company.id if self.company else None)
+
+    @classmethod
+    def default_currency(cls, **pattern):
+        pool = Pool()
+        Company = pool.get('company.company')
+        company = pattern.get('company')
+        if not company:
+            company = cls.default_company()
+        if company:
+            return Company(company).currency.id
+
     @staticmethod
     def default_employee():
         return Transaction().context.get('employee')
 
     @classmethod
-    def default_payment_term(cls):
-        PaymentTerm = Pool().get('account.invoice.payment_term')
-        payment_terms = PaymentTerm.search(cls.payment_term.domain)
-        if len(payment_terms) == 1:
-            return payment_terms[0].id
+    def default_payment_term(cls, **pattern):
+        pool = Pool()
+        Configuration = pool.get('account.configuration')
+        config = Configuration(1)
+        payment_term = config.get_multivalue(
+            'default_customer_payment_term', **pattern)
+        return payment_term.id if payment_term else None
+
+    def get_rec_name(self, name):
+        items = [self.number]
+        if self.reference:
+            items.append(f'[{self.reference}]')
+        return ' '.join(items)
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        _, operator, value = clause
+        if operator.startswith('!') or operator.startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        domain = [bool_op,
+            ('number', operator, value),
+            ('reference', operator, value),
+            ]
+        return domain
 
     @classmethod
     def view_attributes(cls):
@@ -294,24 +326,16 @@ class SaleOpportunity(
         default.setdefault('converted_by')
         return super(SaleOpportunity, cls).copy(opportunities, default=default)
 
-    def get_currency(self, name):
-        return self.company.currency.id
-
-    def get_currency_digits(self, name):
-        return self.company.currency.digits
-
-    @fields.depends('company')
-    def on_change_company(self):
-        if self.company:
-            self.currency = self.company.currency
-            self.currency_digits = self.company.currency.digits
-
-    @fields.depends('party')
+    @fields.depends('party', 'amount', 'company')
     def on_change_party(self):
-        if self.party and self.party.customer_payment_term:
-            self.payment_term = self.party.customer_payment_term
-        else:
-            self.payment_term = self.default_payment_term()
+        self.payment_term = self.default_payment_term(
+            company=self.company.id if self.company else None)
+        if self.party:
+            if self.party.customer_payment_term:
+                self.payment_term = self.party.customer_payment_term
+            if not self.amount:
+                if self.party.customer_currency:
+                    self.currency = self.party.customer_currency
 
     def _get_sale_opportunity(self):
         '''
@@ -326,7 +350,7 @@ class SaleOpportunity(
             company=self.company,
             invoice_address=self.address,
             shipment_address=self.address,
-            currency=self.company.currency,
+            currency=self.currency,
             comment=self.comment,
             sale_date=None,
             origin=self,
@@ -383,7 +407,8 @@ class SaleOpportunity(
     def is_forecast(self):
         pool = Pool()
         Date = pool.get('ir.date')
-        today = Date.today()
+        with Transaction().set_context(company=self.company.id):
+            today = Date.today()
         return self.end_date or datetime.date.max > today
 
     @classmethod
@@ -391,29 +416,44 @@ class SaleOpportunity(
     def won(cls, opportunities):
         pool = Pool()
         Date = pool.get('ir.date')
-        cls.write([o for o in opportunities if o.is_forecast], {
-                'end_date': Date.today(),
-                'state': 'won',
-                })
+        for company, c_opportunities in groupby(
+                opportunities, key=lambda o: o.company):
+            with Transaction().set_context(company=company.id):
+                today = Date.today()
+            cls.write([o for o in c_opportunities if o.is_forecast], {
+                    'end_date': today,
+                    'state': 'won',
+                    })
 
     @classmethod
     @ModelView.button
     @Workflow.transition('lost')
     def lost(cls, opportunities):
-        Date = Pool().get('ir.date')
-        cls.write([o for o in opportunities if o.is_forecast], {
-                'end_date': Date.today(),
-                'state': 'lost',
-                })
+        pool = Pool()
+        Date = pool.get('ir.date')
+        for company, c_opportunities in groupby(
+                opportunities, key=lambda o: o.company):
+            with Transaction().set_context(company=company.id):
+                today = Date.today()
+            cls.write([o for o in c_opportunities if o.is_forecast], {
+                    'end_date': today,
+                    'state': 'lost',
+                    })
 
     @classmethod
     @ModelView.button
     @Workflow.transition('cancelled')
     def cancel(cls, opportunities):
-        Date = Pool().get('ir.date')
-        cls.write([o for o in opportunities if o.is_forecast], {
-                'end_date': Date.today(),
-                })
+        pool = Pool()
+        Date = pool.get('ir.date')
+        for company, c_opportunities in groupby(
+                opportunities, key=lambda o: o.company):
+            with Transaction().set_context(company=company.id):
+                today = Date.today()
+            cls.write([o for o in c_opportunities if o.is_forecast], {
+                    'end_date': today,
+                    'state': 'cancelled',
+                    })
 
     @staticmethod
     def _sale_won_states():
@@ -485,28 +525,29 @@ class SaleOpportunityLine(sequence_ordered(), ModelSQL, ModelView):
         'readonly': Eval('opportunity_state').in_(
             ['converted', 'won', 'lost', 'cancelled']),
         }
-    _depends = ['opportunity_state']
 
-    opportunity = fields.Many2One('sale.opportunity', 'Opportunity',
-        ondelete='CASCADE', select=True, required=True,
+    opportunity = fields.Many2One(
+        'sale.opportunity', "Opportunity", ondelete='CASCADE', required=True,
         states={
             'readonly': _states['readonly'] & Bool(Eval('opportunity')),
-            },
-        depends=_depends)
+            })
     opportunity_state = fields.Function(
         fields.Selection('get_opportunity_states', "Opportunity State"),
         'on_change_with_opportunity_state')
     product = fields.Many2One('product.product', 'Product', required=True,
-        domain=[('salable', '=', True)], states=_states, depends=_depends)
-    quantity = fields.Float('Quantity', required=True,
-        digits=(16, Eval('unit_digits', 2)),
-        states=_states, depends=['unit_digits'] + _depends)
+        domain=[
+            If(Eval('opportunity_state').in_(['lead', 'opportunity'])
+                & ~(Eval('quantity', 0) < 0),
+                ('salable', '=', True),
+                ()),
+            ],
+        states=_states)
+    quantity = fields.Float(
+        "Quantity", digits='unit', required=True, states=_states)
     unit = fields.Many2One('product.uom', 'Unit', required=True,
-        states=_states, depends=_depends)
-    unit_digits = fields.Function(fields.Integer('Unit Digits'),
-        'on_change_with_unit_digits')
+        states=_states)
 
-    del _states, _depends
+    del _states
 
     @classmethod
     def __setup__(cls):
@@ -524,12 +565,6 @@ class SaleOpportunityLine(sequence_ordered(), ModelSQL, ModelView):
         if self.opportunity:
             return self.opportunity.state
 
-    @fields.depends('unit')
-    def on_change_with_unit_digits(self, name=None):
-        if self.unit:
-            return self.unit.digits
-        return 2
-
     @fields.depends('product', 'unit')
     def on_change_product(self):
         if not self.product:
@@ -538,7 +573,6 @@ class SaleOpportunityLine(sequence_ordered(), ModelSQL, ModelView):
         category = self.product.sale_uom.category
         if not self.unit or self.unit.category != category:
             self.unit = self.product.sale_uom
-            self.unit_digits = self.product.sale_uom.digits
 
     def get_sale_line(self, sale):
         '''
@@ -564,234 +598,10 @@ class SaleOpportunityLine(sequence_ordered(), ModelSQL, ModelView):
         pool = Pool()
         Lang = pool.get('ir.lang')
         lang = Lang.get()
-        return (lang.format(
-                '%.*f', (self.unit.digits, self.quantity or 0))
-            + '%s %s @ %s' % (
-                self.unit.symbol, self.product.rec_name,
-                self.opportunity.rec_name))
+        return (lang.format_number_symbol(
+                self.quantity or 0, self.unit, digits=self.unit.digits)
+            + ' %s @ %s' % (self.product.rec_name, self.opportunity.rec_name))
 
     @classmethod
     def search_rec_name(cls, name, clause):
         return [('product.rec_name',) + tuple(clause[1:])]
-
-
-class SaleOpportunityReportMixin:
-    __slots__ = ()
-    number = fields.Integer('Number')
-    converted = fields.Integer('Converted')
-    conversion_rate = fields.Function(fields.Float('Conversion Rate',
-        digits=(1, 4)), 'get_conversion_rate')
-    won = fields.Integer('Won')
-    winning_rate = fields.Function(fields.Float('Winning Rate', digits=(1, 4)),
-        'get_winning_rate')
-    lost = fields.Integer('Lost')
-    company = fields.Many2One('company.company', 'Company')
-    currency = fields.Function(fields.Many2One('currency.currency',
-        'Currency'), 'get_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-            'get_currency_digits')
-    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'])
-    converted_amount = fields.Numeric('Converted Amount',
-            digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'])
-    conversion_amount_rate = fields.Function(fields.Float(
-        'Conversion Amount Rate', digits=(1, 4)), 'get_conversion_amount_rate')
-    won_amount = fields.Numeric('Won Amount',
-        digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    winning_amount_rate = fields.Function(fields.Float(
-            'Winning Amount Rate', digits=(1, 4)), 'get_winning_amount_rate')
-
-    @staticmethod
-    def _converted_state():
-        return ['converted', 'won']
-
-    @staticmethod
-    def _won_state():
-        return ['won']
-
-    @staticmethod
-    def _lost_state():
-        return ['lost']
-
-    def get_conversion_rate(self, name):
-        if self.number:
-            digits = getattr(self.__class__, name).digits[1]
-            return round(float(self.converted) / self.number, digits)
-        else:
-            return 0.0
-
-    def get_winning_rate(self, name):
-        if self.number:
-            digits = getattr(self.__class__, name).digits[1]
-            return round(float(self.won) / self.number, digits)
-        else:
-            return 0.0
-
-    def get_currency(self, name):
-        return self.company.currency.id
-
-    def get_currency_digits(self, name):
-        return self.company.currency.digits
-
-    def get_conversion_amount_rate(self, name):
-        if self.amount:
-            digits = getattr(self.__class__, name).digits[1]
-            return round(
-                float(self.converted_amount) / float(self.amount), digits)
-        else:
-            return 0.0
-
-    def get_winning_amount_rate(self, name):
-        if self.amount:
-            digits = getattr(self.__class__, name).digits[1]
-            return round(float(self.won_amount) / float(self.amount), digits)
-        else:
-            return 0.0
-
-    @classmethod
-    def table_query(cls):
-        Opportunity = Pool().get('sale.opportunity')
-        opportunity = Opportunity.__table__()
-        return opportunity.select(
-            Max(opportunity.create_uid).as_('create_uid'),
-            Max(opportunity.create_date).as_('create_date'),
-            Max(opportunity.write_uid).as_('write_uid'),
-            Max(opportunity.write_date).as_('write_date'),
-            opportunity.company,
-            Count(Literal(1)).as_('number'),
-            Sum(Case(
-                    (opportunity.state.in_(cls._converted_state()),
-                        Literal(1)), else_=Literal(0))).as_('converted'),
-            Sum(Case(
-                    (opportunity.state.in_(cls._won_state()),
-                        Literal(1)), else_=Literal(0))).as_('won'),
-            Sum(Case(
-                    (opportunity.state.in_(cls._lost_state()),
-                        Literal(1)), else_=Literal(0))).as_('lost'),
-            Sum(opportunity.amount).as_('amount'),
-            Sum(Case(
-                    (opportunity.state.in_(cls._converted_state()),
-                        opportunity.amount),
-                    else_=Literal(0))).as_('converted_amount'),
-            Sum(Case(
-                    (opportunity.state.in_(cls._won_state()),
-                        opportunity.amount),
-                    else_=Literal(0))).as_('won_amount'))
-
-
-class SaleOpportunityEmployee(SaleOpportunityReportMixin, ModelSQL, ModelView):
-    'Sale Opportunity per Employee'
-    __name__ = 'sale.opportunity_employee'
-    employee = fields.Many2One('company.employee', 'Employee')
-
-    @classmethod
-    def table_query(cls):
-        query = super(SaleOpportunityEmployee, cls).table_query()
-        opportunity, = query.from_
-        query.columns += (
-            Coalesce(opportunity.employee, 0).as_('id'),
-            opportunity.employee,
-            )
-        where = Literal(True)
-        if Transaction().context.get('start_date'):
-            where &= (opportunity.start_date
-                >= Transaction().context['start_date'])
-        if Transaction().context.get('end_date'):
-            where &= (opportunity.start_date
-                <= Transaction().context['end_date'])
-        query.where = where
-        query.group_by = (opportunity.employee, opportunity.company)
-        return query
-
-
-class SaleOpportunityEmployeeContext(ModelView):
-    'Sale Opportunity per Employee Context'
-    __name__ = 'sale.opportunity_employee.context'
-    start_date = fields.Date('Start Date')
-    end_date = fields.Date('End Date')
-
-
-class SaleOpportunityMonthly(SaleOpportunityReportMixin, ModelSQL, ModelView):
-    'Sale Opportunity per Month'
-    __name__ = 'sale.opportunity_monthly'
-    year = fields.Char('Year')
-    month = fields.Many2One('ir.calendar.month', "Month")
-    year_month = fields.Function(fields.Char('Year-Month'),
-            'get_year_month')
-
-    @classmethod
-    def __setup__(cls):
-        super(SaleOpportunityMonthly, cls).__setup__()
-        cls._order.insert(0, ('year', 'DESC'))
-        cls._order.insert(1, ('month.index', 'DESC'))
-
-    def get_year_month(self, name):
-        return '%s-%s' % (self.year, self.month.index)
-
-    @classmethod
-    def table_query(cls):
-        pool = Pool()
-        Month = pool.get('ir.calendar.month')
-        month = Month.__table__()
-        query = super(SaleOpportunityMonthly, cls).table_query()
-        opportunity, = query.from_
-        type_id = cls.id.sql_type().base
-        type_year = cls.year.sql_type().base
-        year_column = Extract('YEAR', opportunity.start_date
-            ).cast(type_year).as_('year')
-        month_index = Extract('MONTH', opportunity.start_date)
-        query.from_ = opportunity.join(
-            month, condition=month_index == month.index)
-        query.columns += (
-            Max(Extract('MONTH', opportunity.start_date)
-                + Extract('YEAR', opportunity.start_date) * 100
-                ).cast(type_id).as_('id'),
-            year_column,
-            month.id.as_('month'))
-        query.group_by = (year_column, month.id, opportunity.company)
-        return query
-
-
-class SaleOpportunityEmployeeMonthly(
-        SaleOpportunityReportMixin, ModelSQL, ModelView):
-    'Sale Opportunity per Employee per Month'
-    __name__ = 'sale.opportunity_employee_monthly'
-    year = fields.Char('Year')
-    month = fields.Many2One('ir.calendar.month', "Month")
-    employee = fields.Many2One('company.employee', 'Employee')
-
-    @classmethod
-    def __setup__(cls):
-        super(SaleOpportunityEmployeeMonthly, cls).__setup__()
-        cls._order.insert(0, ('year', 'DESC'))
-        cls._order.insert(1, ('month.index', 'DESC'))
-        cls._order.insert(2, ('employee', 'ASC'))
-
-    @classmethod
-    def table_query(cls):
-        pool = Pool()
-        Month = pool.get('ir.calendar.month')
-        month = Month.__table__()
-        query = super(SaleOpportunityEmployeeMonthly, cls).table_query()
-        opportunity, = query.from_
-        type_id = cls.id.sql_type().base
-        type_year = cls.year.sql_type().base
-        year_column = Extract('YEAR', opportunity.start_date
-            ).cast(type_year).as_('year')
-        month_index = Extract('MONTH', opportunity.start_date)
-        query.from_ = opportunity.join(
-            month, condition=month_index == month.index)
-        query.columns += (
-            Max(Extract('MONTH', opportunity.start_date)
-                + Extract('YEAR', opportunity.start_date) * 100
-                + Coalesce(opportunity.employee, 0) * 1000000
-                ).cast(type_id).as_('id'),
-            year_column,
-            month.id.as_('month'),
-            opportunity.employee,
-            )
-        query.group_by = (year_column, month.id,
-            opportunity.employee, opportunity.company)
-        return query
